@@ -1,10 +1,10 @@
 from __future__ import annotations
-from .format import check_nname, nname_formatting
+from .name_strings import note_name_formatting, chord_name_formatting
+from .chord_names import chord_names
 import numpy as np
-from scipy import signal
+import scipy as sp
 import IPython
-from typing import Union, Callable
-import re
+from typing import Optional, Union, Callable
 
 
 NUM_C0 = 12 # MIDI note number of C0
@@ -27,9 +27,15 @@ class Note:
         Single note. It can be initialized with note name or MIDI note number.
 
         Args:
-            query (Union[str, int]): string of note name or midi note number
-            octave (int, optional): octave of the note.
-            A4 (float, optional): tuning. freqency of A4.
+            query (Union[str, int]):
+                string of note name or midi note number. If string is given,
+                valid string pattern is '[A-Ga-g][#♯+b♭-]?\d*'.
+                Ex: 'C', 'C#', 'Cb', 'C4'
+            octave (int, optional):
+                octave of the note. This argument is ignored if octave is
+                specified in the note name string, or if query is int.
+            A4 (float, optional):
+                tuning. freqency of A4.
 
         \Attributes:
             - name (str): note name
@@ -41,14 +47,25 @@ class Note:
 
         Examples:
             >>> import munotes as mn
+            >>> note = mn.Note("C4")
+            >>> print(note)
+            C4
+
+            You can also input note name and octave separately.
+
             >>> note = mn.Note("C", 4)
             >>> print(note)
             C4
+
+            You can also input MIDI note number as an integer.
+
+            >>> note = mn.Note(60)
+            >>> print(note)
+            C4
         """
         if isinstance(query, str):
-            self.name = check_nname(query, return_nname=True)
+            self.name, self._octave = note_name_formatting(query, octave)
             self._idx = self._return_name_idx()
-            self._octave = octave
             self._num = NUM_C0 + 12*self.octave + self.idx
         elif isinstance(query, int):
             assert 0 <= query <= 127, "MIDI note number must be in 0 ~ 127"
@@ -126,7 +143,7 @@ class Note:
             np.ndarray: sin wave of the note
 
         Examples:
-            >>> note = mn.Note("C", 4)
+            >>> note = mn.Note("C4")
             >>> note.sin()
             array([ 0.        ,  0.07448499,  0.14855616, ..., -0.59706869,
                    -0.65516123, -0.70961388])
@@ -134,21 +151,38 @@ class Note:
         t = self._return_time_axis(sec, sr)
         return np.sum(np.sin(t), axis=0)
 
-    def square(self, sec: float = 1., sr: int = 22050) -> np.ndarray:
-        """Generate square wave of the note"""
+    def square(
+        self,
+        sec: float = 1.,
+        sr: int = 22050,
+        duty: float = 0.5
+    ) -> np.ndarray:
+        """
+        Generate square wave of the note with scipy.signal.square.
+        kwargs of scipy.signal.square are supported.
+        """
         t = self._return_time_axis(sec, sr)
-        return np.sum(signal.square(t), axis=0)
+        return np.sum(sp.signal.square(t, duty), axis=0)
 
-    def sawtooth(self, sec: float = 1., sr: int = 22050) -> np.ndarray:
-        """Generate sawtooth wave of the note"""
+    def sawtooth(
+        self,
+        sec: float = 1.,
+        sr: int = 22050,
+        width: float = 1.
+    ) -> np.ndarray:
+        """
+        Generate sawtooth wave of the note with scipy.signal.sawtooth.
+        kwargs of scipy.signal.sawtooth are supported.
+        """
         t = self._return_time_axis(sec, sr)
-        return np.sum(signal.sawtooth(t), axis=0)
+        return np.sum(sp.signal.sawtooth(t, width), axis=0)
 
     def render(
         self,
         waveform: Union[str, Callable] = 'sin',
         sec: float = 1.,
-        sr: int = 22050
+        sr: int = 22050,
+        **kwargs
     ) -> np.ndarray:
         """
         Rendering waveform of the note.
@@ -165,12 +199,16 @@ class Note:
                 duration in seconds.
             sr (int, optional):
                 sampling rate.
+            **kwargs (optional):
+                keyword arguments for waveform function. 'duty' for
+                'square', 'width' for 'sawtooth' and any args for
+                user-defined waveform function are supported.
 
         Returns:
             np.ndarray: waveform of the note
 
         Examples:
-            >>> note = mn.Note("C", 4)
+            >>> note = mn.Note("C4")
             >>> note.render('sin')
             array([ 0.        ,  0.07448499,  0.14855616, ..., -0.59706869,
                    -0.65516123, -0.70961388])
@@ -187,10 +225,10 @@ class Note:
         if isinstance(waveform, str):
             assert waveform in SUPPOERTED_WAVEFORMS, \
                 f"waveform string must be in {SUPPOERTED_WAVEFORMS}"
-            return getattr(self, waveform)(sec, sr)
+            return getattr(self, waveform)(sec, sr, **kwargs)
         else:
             t = self._return_time_axis(sec, sr)
-            return np.sum(waveform(t), axis=0)
+            return np.sum(waveform(t, **kwargs), axis=0)
 
     def _return_time_axis(self, sec: float, sr: int) -> np.ndarray:
         """
@@ -210,7 +248,8 @@ class Note:
     def play(
         self,
         waveform: Union[str, Callable] = 'sin',
-        sec: float = 1.
+        sec: float = 1.,
+        **kwargs
     ) -> IPython.display.Audio:
         """
         Play note sound in IPython notebook.
@@ -225,7 +264,7 @@ class Note:
         Returns:
             IPython.display.Audio: audio object
         """
-        y = self.render(waveform, sec, PLAY_SR)
+        y = self.render(waveform, sec, PLAY_SR, **kwargs)
         return IPython.display.Audio(y, rate=PLAY_SR)
 
 
@@ -237,7 +276,7 @@ class Note:
             n_semitones (int): number of semitones to transpose
 
         Examples:
-            >>> note = mn.Note("C", 4)
+            >>> note = mn.Note("C4")
             >>> note.transpose(1)
             >>> print(note)
             C#4
@@ -258,14 +297,14 @@ class Note:
             stand_A4 (bool, optional): if True, the tuning standard is A4.
 
         Examples:
-            >>> note = mn.Note("C", 4)
+            >>> note = mn.Note("C4")
             >>> print(note.freq)
             >>> note.tuning(270.)
             >>> print(note.freq)
             261.6255653005986
             270.0
 
-            >>> note = mn.Note("C", 4)
+            >>> note = mn.Note("C4")
             >>> print(note.freq)
             >>> note.tuning(450., stand_A4=True)
             >>> print(note.freq)
@@ -364,7 +403,7 @@ class Rest(Note):
     def sawtooth(self, sec=1., sr=22050) -> np.ndarray:
         return np.zeros(int(sr*sec))
 
-    def render(self, waveform=None, sec=1., sr=22050) -> np.ndarray:
+    def render(self, waveform=None, sec=1., sr=22050, **kwargs) -> np.ndarray:
         return np.zeros(int(sr*sec))
 
     def transpose(self, *args, **kwargs):
@@ -382,7 +421,7 @@ class Rest(Note):
 
 
 class Notes(Note):
-    def __init__(self, *notes: Union[Note, int], A4: float = 440.):
+    def __init__(self, *notes: Union[Note, int, str], A4: float = 440.):
         """
         Notes class. Manage multiple notes at once.
 
@@ -390,9 +429,8 @@ class Notes(Note):
             *notes (Union[Note, int]):
                 notes.
                 supported input types:
-                    - Note
-                    - Notes
-                    - Chord
+                    - Note (mn.Note, mn.Notes, etc.)
+                    - str (note name)
                     - int (midi note number)
             A4 (float, optional):
                 tuning. frequency of A4.
@@ -418,29 +456,31 @@ class Notes(Note):
         Examples:
             >>> import musicnote as mn
             >>> notes = mn.Notes(
-            >>>     mn.Note("C", 4),
-            >>>     mn.Note("E", 4),
-            >>>     mn.Note("G", 4)
+            >>>     mn.Note("C4"),
+            >>>     mn.Note("E4"),
+            >>>     mn.Note("G4")
             >>> )
             >>> notes
             Notes [Note C4, Note E4, Note G4]
 
+            >>> notes = mn.Notes("C4", "E4", "G4")
+            >>> notes
+            Notes [Note C4, Note E4, Note G4]
+
             >>> notes = mn.Notes(60, 64, 67)
             >>> notes
             Notes [Note C4, Note E4, Note G4]
 
-            >>> notes = mn.Notes(60, 64, 67) + mn.Note("C", 5)
+            >>> notes = mn.Notes(60, 64, 67) + mn.Note("C5")
             >>> notes
             Notes [Note C4, Note E4, Note G4, Note C5]
         """
         assert notes, "Notes must be input"
         notes_ = []
         for note in notes:
-            if isinstance(note, Notes):
-                notes_ += note.notes
-            elif isinstance(note, Note):
-                notes_.append(note)
-            elif isinstance(note, int):
+            if isinstance(note, Note):
+                notes_ += note._notes
+            elif isinstance(note, (str, int)):
                 notes_.append(Note(note))
             else:
                 raise ValueError(f"Unsupported type: '{type(note)}'")
@@ -486,8 +526,8 @@ class Notes(Note):
             note (Union[Note, int]): Note (or Notes or Chord) or midi note number
 
         Examples:
-            >>> notes = mn.Notes(mn.Note("C", 4))
-            >>> notes.append(mn.Note("E", 4), mn.Note("G", 4))
+            >>> notes = mn.Notes(mn.Note("C4"))
+            >>> notes.append(mn.Note("E4"), mn.Note("G4"))
             >>> notes
             Notes [Note C4, Note E4, Note G4]
         """
@@ -514,32 +554,30 @@ class Notes(Note):
 
 
 
-chord_intervals = {
-    "": (0,4,7),
-    "m": (0,3,7),
-    "7": (0,4,7,10),
-    "m7": (0,3,7,10),
-    "M7": (0,4,7,11),
-    "mM7": (0,3,7,11),
-    "sus4": (0,5,7),
-    "dim": (0,3,6),
-    "dim7": (0,3,6,9),
-    "aug": (0,4,8),
-    "6": (0,4,7,9),
-    "m6": (0,3,7,9),
-    "sus2": (0,2,7)
-}
-PITCH_PATTERN = '[A-G][#, b]*'
-
 class Chord(Notes):
-    def __init__(self, chord_name: str, octave: int = 4, A4: float = 440.):
+    def __init__(
+        self,
+        chord_name: str,
+        type: Optional[str] = None,
+        octave: int = 4,
+        A4: float = 440.
+    ):
         """
         Chord class.
         Estimating notes from chord names and creating a Notes object.
 
+        Supported chord names are shown in ``mn.chord_names``.
+        ``mn.chord_names`` is a dictionary of chord names and intervals
+        between notes with the root note as 0.
+        Ex: {'': (0, 4, 7), 'm': (0, 3, 7), 'dim7': (0, 3, 6, 9), ...}.
+        You can add your own chord names and intervals to this dictionary.
+
         Args:
             chord_name (str):
-                chord name string
+                string of chord name. Chord type in the string is ignored
+                if 'type' argument is specified.
+            type (str, optional):
+                chord type. Ex. '', 'm7', '7', 'sus4'.
             octave (int, optional):
                 octave of the root note to initialize notes.
             A4 (float, optional):
@@ -560,16 +598,22 @@ class Chord(Notes):
             >>> chord = mn.Chord("C")
             >>> chord.names
             ['C', 'E', 'G']
-        """
-        chord_name = nname_formatting(chord_name)
-        pitch_search = re.match(PITCH_PATTERN, chord_name)
-        assert pitch_search, f"'{chord_name}' is an invalid string"
 
-        border = pitch_search.end()
-        root_name, type = chord_name[:border], chord_name[border:]
+            Adding arbitrary chords.
+
+            >>> mn.chord_names["black"] = (0, 1, 2, 3, 4)
+            >>> chord = mn.Chord("C", "black")
+            >>> chord.names
+            ['C', 'C#', 'D', 'D#', 'E']
+
+        Note:
+            Unlike Note, the argument value takes precedence over the
+            input string.
+        """
+        root_name, type = chord_name_formatting(chord_name, type)
         root = Note(root_name, octave)
         name = root.name + type
-        interval = chord_intervals[type]
+        interval = chord_names[type]
 
         self.name = name
         self._type = type
